/*-
 * FIPS 180-2 SHA-224/256/384/512 implementation
 * Last update: 02/21/2006
 * Issue date:  01/21/2006
 *
 * Copyright (C) 2007 Michael Reynolds <d-dot-sha256-at-delinked-dot-us>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#pragma check_stack(off)
#define _CRT_SECURE_NO_DEPRECATE
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "sha2.h"

#define BUFSIZE 900

typedef struct {
	DWORD  mVersion;
	HWND   mHwnd;
	BOOL   mKeep;
} LOADINFO;

extern "C" __declspec(dllexport) void __stdcall LoadDll(LOADINFO* l)
{
	l->mKeep = FALSE;
}

extern "C" __declspec(dllexport) int __stdcall UnloadDll(int)
{
	return 1;
}

extern "C" int WINAPI SHA224(HWND,HWND,char *mIRCdata,char *,BOOL,BOOL)
{
	if (*mIRCdata == '\0')
	{
		lstrcpy(mIRCdata,"$false");
		return 3;
	}
	unsigned int len = strlen(mIRCdata);
	char enc_out[SHA224_DIGEST_SIZE*2+1];
	unsigned char enc[SHA224_DIGEST_SIZE+1];
	unsigned char *ePtr = enc;
	char *eoPtr = enc_out;
	sha224((unsigned char *)mIRCdata, len, ePtr);
	int i;
	enc_out[2*SHA224_DIGEST_SIZE] = 0;
	for (i = 0; i < SHA224_DIGEST_SIZE; i++) {
		wsprintf(enc_out + 2 * i, "%02x", ePtr[i]);
	}
	lstrcpy(mIRCdata,eoPtr);
	return 3;
}

extern "C" int WINAPI SHA256(HWND,HWND,char *mIRCdata,char *,BOOL,BOOL)
{
	if (*mIRCdata == '\0')
	{
		lstrcpy(mIRCdata,"$false");
		return 3;
	}
	unsigned int len = strlen(mIRCdata);
	char enc_out[SHA256_DIGEST_SIZE*2+1];
	unsigned char enc[SHA256_DIGEST_SIZE+1];
	unsigned char *ePtr = enc;
	//char *eoPtr = enc_out;
	sha256((unsigned char *)mIRCdata, len, ePtr);
	int i;
	enc_out[2*SHA256_DIGEST_SIZE] = 0;
	for (i = 0; i < SHA256_DIGEST_SIZE; i++) {
		wsprintf(enc_out + 2 * i, "%02x", enc[i]);
	}
	lstrcpy(mIRCdata,enc_out);
	return 3;
}

extern "C" int WINAPI SHA384(HWND,HWND,char *mIRCdata,char *,BOOL,BOOL)
{
	if (*mIRCdata == '\0')
	{
		lstrcpy(mIRCdata,"$false");
		return 3;
	}
	unsigned int len = strlen(mIRCdata);
	char enc_out[SHA384_DIGEST_SIZE*2+1];
	unsigned char enc[SHA384_DIGEST_SIZE+1];
	unsigned char *ePtr = enc;
	char *eoPtr = enc_out;
	sha384((unsigned char *)mIRCdata, len, ePtr);
	int i;
	enc_out[2*SHA384_DIGEST_SIZE] = 0;
	for (i = 0; i < SHA384_DIGEST_SIZE; i++) {
		wsprintf(enc_out + 2 * i, "%02x", ePtr[i]);
	}
	lstrcpy(mIRCdata,eoPtr);
	return 3;
}

extern "C" int WINAPI SHA512(HWND,HWND,char *mIRCdata,char *,BOOL,BOOL)
{
	if (*mIRCdata == '\0')
	{
		lstrcpy(mIRCdata,"$false");
		return 3;
	}
	unsigned int len = strlen(mIRCdata);
	char enc_out[SHA512_DIGEST_SIZE*2+1];
	unsigned char enc[SHA512_DIGEST_SIZE+1];
	unsigned char *ePtr = enc;
	char *eoPtr = enc_out;
	sha512((unsigned char *)mIRCdata, len, ePtr);
	int i;
	enc_out[2*SHA512_DIGEST_SIZE] = 0;
	for (i = 0; i < SHA512_DIGEST_SIZE; i++) {
		wsprintf(enc_out + 2 * i, "%02x", ePtr[i]);
	}
	lstrcpy(mIRCdata,eoPtr);
	return 3;
}

extern "C" int WINAPI DLLInfo(HWND,HWND,char *mIRCdata,char *,BOOL,BOOL)
{
	wsprintf(mIRCdata,"mIRC SHA256 1.0 by wshs.");
	return 3;
}
